@@ -9,8 +9,11 @@
 
 """Infrastructure to create test scenarios in the database."""
 
+import contextlib
 import tempfile
 import textwrap
+import time
+from collections.abc import Generator
 from dataclasses import dataclass
 from datetime import timedelta
 from pathlib import Path
@@ -32,6 +35,9 @@ from debusine.artifacts.models import (
     BaseArtifactDataModel,
     CollectionCategory,
     DebianPackageBuildLog,
+    DebusineSigningInput,
+    DebusineSigningOutput,
+    SigningResult,
 )
 from debusine.artifacts.playground import ArtifactPlayground
 from debusine.assets import (
@@ -74,7 +80,7 @@ from debusine.db.models import (
     Workspace,
 )
 from debusine.db.models.permissions import (
-    Roles,
+    PermissionUser,
     get_resource_scope,
     resolve_role,
 )
@@ -93,6 +99,7 @@ from debusine.server.workflows.models import (
     WorkRequestWorkflowData,
 )
 from debusine.tasks.models import (
+    ActionUpdateCollectionWithArtifacts,
     BackendType,
     BaseTaskData,
     SbuildBuildComponent,
@@ -265,7 +272,7 @@ class Playground(ArtifactPlayground):
     def create_group_role(
         self,
         resource: Model,
-        role: Roles | str,
+        role: str,
         users: list[User] | None = None,
         name: str | None = None,
     ) -> Group:
@@ -275,7 +282,7 @@ class Playground(ArtifactPlayground):
         if users is None:
             users = []
 
-        role = resolve_role(getattr(resource, "Roles"), role)
+        role = resolve_role(resource, role)
         group_scope = get_resource_scope(resource)
         group = self.create_group(name, users, scope=group_scope)
         roles_model = getattr(resource.__class__, "objects").get_roles_model()
@@ -283,6 +290,28 @@ class Playground(ArtifactPlayground):
         group.assign_role(resource, role)
         return group
 
+    @contextlib.contextmanager
+    def assign_role(
+        self, resource: Model, user: PermissionUser, *roles: str
+    ) -> Generator[None, None, None]:
+        """Temporarily assign one or more roles to the user on the resource."""
+        groups: list[Group] = []
+
+        if roles:
+            assert user and user.is_authenticated
+            for role in roles:
+                name = f"test_assign_role_{role}_{time.monotonic_ns()}"
+                group = self.create_group_role(
+                    resource, role, [user], name=name
+                )
+                groups.append(group)
+
+        try:
+            yield
+        finally:
+            for group in groups:
+                group.delete()
+
     @context.disable_permission_checks()
     def add_user(
         self, group: Group, user: User, role: Group.Roles = Group.Roles.MEMBER
@@ -439,10 +468,12 @@ class Playground(ArtifactPlayground):
         self,
         task_name: str | WorkflowTemplate = "noop",
         task_data: BaseWorkflowData | dict[str, Any] | None = None,
+        *,
         parent: WorkRequest | None = None,
         status: WorkRequest.Statuses | None = None,
         created_by: User | None = None,
         validate: bool = True,
+        workspace: Workspace | None = None,
     ) -> WorkRequest:
         """Create a workflow."""
         if created_by is None:
@@ -458,7 +489,9 @@ class Playground(ArtifactPlayground):
                 )
             except WorkflowTemplate.DoesNotExist:
                 template = self.create_workflow_template(
-                    name=workflow_template_name, task_name=task_name
+                    name=workflow_template_name,
+                    task_name=task_name,
+                    workspace=workspace,
                 )
 
         if isinstance(task_data, BaseWorkflowData):
@@ -490,6 +523,7 @@ class Playground(ArtifactPlayground):
         workspace: Workspace | None = None,
         task_name: str = "noop",
         task_data: BaseTaskData | dict[str, Any] | None = None,
+        assign_contributor_role: bool = False,
         **kwargs: Any,
     ) -> WorkRequest:
         """
@@ -500,6 +534,8 @@ class Playground(ArtifactPlayground):
         :param result: if not None call mark_completed(result)
         :param expired: True to create an expired work request
           (created_at: 1 year ago, expiration_delay: 1 day)
+        :param assign_contributor_role: assign the ``CONTRIBUTOR`` role on
+          the workspace to the creator of the work request
         :param kwargs: use them when creating the WorkRequest model
         """
         # FileStore and Workspace are usually created by the migrations
@@ -516,8 +552,7 @@ class Playground(ArtifactPlayground):
             "workspace": workspace,
         }
         if "created_by" not in kwargs:
-            token = self.create_user_token()
-            defaults["created_by"] = token.user
+            defaults["created_by"] = self.get_default_user()
         if expired:
             defaults.update(
                 created_at=timezone.now() - timedelta(days=365),
@@ -542,6 +577,13 @@ class Playground(ArtifactPlayground):
             work_request.created_at = created_at
             work_request.save()
 
+        if assign_contributor_role:
+            self.create_group_role(
+                work_request.workspace,
+                Workspace.Roles.CONTRIBUTOR,
+                users=[work_request.created_by],
+            )
+
         if assign_new_worker:
             work_request.assign_worker(self.create_worker())
 
@@ -872,9 +914,11 @@ class Playground(ArtifactPlayground):
         self,
         *,
         name: str = "hello",
+        binaries: list[str] | None = None,
         version: str = "1.0-1",
         architectures: set[str] | None = None,
         workspace: Workspace | None = None,
+        work_request: WorkRequest | None = None,
         created_by: User | None = None,
         create_files: bool = False,
     ) -> Artifact:
@@ -882,11 +926,16 @@ class Playground(ArtifactPlayground):
         with tempfile.TemporaryDirectory() as tempdir:
             workdir = Path(tempdir)
             source_package = self.create_source_package(
-                workdir, name=name, version=version, architectures=architectures
+                workdir,
+                name=name,
+                binaries=binaries,
+                version=version,
+                architectures=architectures,
             )
             return self.create_artifact_from_local(
                 source_package,
                 workspace=workspace,
+                work_request=work_request,
                 created_by=created_by,
                 create_files=create_files,
             )
@@ -944,8 +993,11 @@ class Playground(ArtifactPlayground):
         source: Literal[True] = True,
         binary: Literal[False],
         binaries: list[str] | None = ...,
+        bin_architecture: str = ...,
         workspace: Workspace | None = ...,
+        work_request: WorkRequest | None = None,
         created_by: User | None = ...,
+        create_files: bool = ...,
     ) -> SourceUploadArtifacts: ...
 
     @overload
@@ -957,8 +1009,11 @@ class Playground(ArtifactPlayground):
         source: Literal[False],
         binary: Literal[True] = True,
         binaries: list[str] | None = ...,
+        bin_architecture: str = ...,
         workspace: Workspace | None = ...,
+        work_request: WorkRequest | None = None,
         created_by: User | None = ...,
+        create_files: bool = ...,
     ) -> BinaryUploadArtifacts: ...
 
     @overload
@@ -970,8 +1025,11 @@ class Playground(ArtifactPlayground):
         source: Literal[True] = True,
         binary: Literal[True] = True,
         binaries: list[str] | None = ...,
+        bin_architecture: str = ...,
         workspace: Workspace | None = ...,
+        work_request: WorkRequest | None = None,
         created_by: User | None = ...,
+        create_files: bool = ...,
     ) -> MixedUploadArtifacts: ...
 
     @context.disable_permission_checks()
@@ -983,7 +1041,9 @@ class Playground(ArtifactPlayground):
         source: bool = True,
         binary: bool = True,
         binaries: list[str] | None = None,
+        bin_architecture: str = "all",
         workspace: Workspace | None = None,
+        work_request: WorkRequest | None = None,
         created_by: User | None = None,
         create_files: bool = False,
     ) -> UploadArtifacts:
@@ -1004,6 +1064,7 @@ class Playground(ArtifactPlayground):
                 source=source,
                 binary=binary,
                 binaries=binaries,
+                bin_architecture=bin_architecture,
             )
             local_artifacts["upload"] = [upload]
             if source:
@@ -1030,6 +1091,7 @@ class Playground(ArtifactPlayground):
                     self.create_artifact_from_local(
                         local_artifact,
                         workspace=workspace,
+                        work_request=work_request,
                         created_by=created_by,
                         create_files=create_files,
                     )
@@ -1080,7 +1142,10 @@ class Playground(ArtifactPlayground):
             else:
                 created_by = self.get_default_user()
         data = DebianPackageBuildLog(
-            source=source, version=version, filename=filename
+            source=source,
+            version=version,
+            architecture=build_arch,
+            filename=filename,
         )
         artifact, _ = self.create_artifact(
             paths={filename: contents},
@@ -1099,15 +1164,40 @@ class Playground(ArtifactPlayground):
         binary_package_name: str | None = "hello",
         trusted_certs: list[str] | None = None,
         workspace: Workspace | None = None,
+        work_request: WorkRequest | None = None,
     ) -> Artifact:
         """Create a minimal `debusine:signing-input` artifact."""
         artifact, _ = self.create_artifact(
             category=ArtifactCategory.SIGNING_INPUT,
-            data={
-                "trusted_certs": trusted_certs,
-                "binary_package_name": binary_package_name,
-            },
+            data=DebusineSigningInput(
+                trusted_certs=trusted_certs,
+                binary_package_name=binary_package_name,
+            ),
             workspace=workspace,
+            work_request=work_request,
+        )
+        return artifact
+
+    def create_signing_output_artifact(
+        self,
+        purpose: KeyPurpose,
+        fingerprint: str,
+        results: list[SigningResult] | None = None,
+        binary_package_name: str | None = None,
+        workspace: Workspace | None = None,
+        work_request: WorkRequest | None = None,
+    ) -> Artifact:
+        """Create a minimal `debusine:signing-output` artifact."""
+        artifact, _ = self.create_artifact(
+            category=ArtifactCategory.SIGNING_OUTPUT,
+            data=DebusineSigningOutput(
+                purpose=purpose,
+                fingerprint=fingerprint,
+                results=results or [],
+                binary_package_name=binary_package_name,
+            ),
+            workspace=workspace,
+            work_request=work_request,
         )
         return artifact
 
@@ -1259,6 +1349,7 @@ class Playground(ArtifactPlayground):
         workspace: Workspace | None = None,
         vendor: str = "Debian",
         mirror: str = "https://deb.debian.org",
+        components: list[str] | None = None,
         pkglist: list[str] | None = None,
         with_init: bool = True,
         with_dev: bool = True,
@@ -1296,6 +1387,8 @@ class Playground(ArtifactPlayground):
                 return lookup_result.collection_item
             except KeyError:
                 pass
+        if components is None:
+            components = ["main"]
         if environment is None:
             # Lookup failed: create it
             data = {
@@ -1304,6 +1397,7 @@ class Playground(ArtifactPlayground):
                 "mirror": mirror,
                 "pkglist": pkglist or [],
                 "codename": codename,
+                "components": components,
                 "architecture": architecture,
                 "variant": variant,
                 "with_init": with_init,
@@ -1319,7 +1413,7 @@ class Playground(ArtifactPlayground):
             variables = {"backend": "unshare", "variant": variant}
 
         try:
-            return CollectionItem.active_objects.get(
+            return CollectionItem.objects.active().get(
                 parent_collection=collection,
                 artifact=environment,
                 data__contains=variables,
@@ -1337,6 +1431,7 @@ class Playground(ArtifactPlayground):
         *,
         source: Artifact,
         architecture: str = "all",
+        host_architecture: str | None = None,
         environment: Artifact,
         workflow: WorkRequest | None = None,
         workspace: Workspace | None = None,
@@ -1344,7 +1439,7 @@ class Playground(ArtifactPlayground):
         """Create a sbuild work request."""
         task_data = SbuildData(
             input=SbuildInput(source_artifact=source.pk),
-            host_architecture=architecture,
+            host_architecture=host_architecture or architecture,
             environment=environment.pk,
             backend=BackendType.UNSHARE,
             build_components=[
@@ -1380,7 +1475,9 @@ class Playground(ArtifactPlayground):
         self,
         source: Artifact,
         *,
+        binary_name: str | None = None,
         architecture: str = "all",
+        host_architecture: str | None = None,
         workflow: WorkRequest | None = None,
         environment: Artifact | None = None,
         worker: Worker | None = None,
@@ -1401,10 +1498,12 @@ class Playground(ArtifactPlayground):
         work_request = self.create_sbuild_work_request(
             source=source,
             architecture=architecture,
+            host_architecture=host_architecture,
             environment=environment,
             workflow=workflow,
             workspace=workspace,
         )
+        work_request.mark_pending()
         if worker is None:
             worker = self.create_worker()
 
@@ -1412,58 +1511,60 @@ class Playground(ArtifactPlayground):
         work_request.assign_worker(worker)
         work_request.mark_running()
 
-        with tempfile.TemporaryDirectory() as tempdir:
-            workdir = Path(tempdir)
+        # Add a build log
+        buildlog_artifact = self.create_build_log_artifact(
+            source=name,
+            version=version,
+            build_arch=build_arch,
+            workspace=workspace,
+            work_request=work_request,
+        )
+        self.create_artifact_relation(
+            buildlog_artifact, source, ArtifactRelation.Relations.RELATES_TO
+        )
 
-            # Add a build log
-            buildlog_artifact = self.create_build_log_artifact(
-                source=name,
-                version=version,
-                build_arch=build_arch,
-                workspace=workspace,
-                work_request=work_request,
-            )
-            self.create_artifact_relation(
-                buildlog_artifact, source, ArtifactRelation.Relations.RELATES_TO
+        # Add the binary upload
+        upload_artifacts = self.create_upload_artifacts(
+            src_name=name,
+            version=version,
+            source=False,
+            binary=True,
+            binaries=[binary_name or name],
+            bin_architecture=architecture,
+            workspace=workspace,
+            work_request=work_request,
+            created_by=work_request.created_by,
+            create_files=True,
+        )
+        assert len(upload_artifacts.binaries) == 1
+        self.create_artifact_relation(
+            upload_artifacts.binaries[0],
+            source,
+            ArtifactRelation.Relations.BUILT_USING,
+        )
+        self.create_artifact_relation(
+            buildlog_artifact,
+            upload_artifacts.binaries[0],
+            ArtifactRelation.Relations.RELATES_TO,
+        )
+
+        if workflow is not None:
+            work_request.add_event_reaction(
+                "on_success",
+                ActionUpdateCollectionWithArtifacts(
+                    collection="internal@collections",
+                    name_template=f"build-{architecture}",
+                    variables={
+                        "binary_names": [binary_name or name],
+                        "architecture": architecture,
+                        "source_package_name": name,
+                    },
+                    artifact_filters={"category": ArtifactCategory.UPLOAD},
+                ),
             )
 
-            # Add the .deb
-            binarypackage = self.create_binary_package(
-                workdir, name=name, version=version, architecture=architecture
-            )
-            binarypackage_artifact = self.create_artifact_from_local(
-                binarypackage,
-                workspace=workspace,
-                work_request=work_request,
-                created_by=work_request.created_by,
-                create_files=True,
-            )
-            self.create_artifact_relation(
-                binarypackage_artifact,
-                source,
-                ArtifactRelation.Relations.BUILT_USING,
-            )
-            self.create_artifact_relation(
-                buildlog_artifact,
-                binarypackage_artifact,
-                ArtifactRelation.Relations.RELATES_TO,
-            )
-
-            # TODO: create the .changes file / Upload artifact
-            # TODO:     adding the .dsc, the sources, the .deb
-            # TODO: self.create_artifact_relation(
-            # TODO:     upload,
-            # TODO:     binarypackage_artifact,
-            # TODO:     ArtifactRelation.Relations.EXTENDS,
-            # TODO: )
-            # TODO: self.create_artifact_relation(
-            # TODO:     upload,
-            # TODO:     binarypackage_artifact,
-            # TODO:     ArtifactRelation.Relations.RELATES_TO,
-            # TODO: )
-
-            # Complete the task
-            work_request.mark_completed(WorkRequest.Results.SUCCESS)
+        # Complete the task
+        work_request.mark_completed(WorkRequest.Results.SUCCESS)
 
         return work_request
 
