@@ -27,32 +27,34 @@ app.autodiscover_tasks()
 # can't fix that here.
 @worker_init.connect  # type: ignore[misc]
 def connect_worker(*, sender: WorkController, **kwargs: Any) -> None:
-    """Set up and connect an appropriate :class:`Worker` when Celery starts."""
+    """Set up and connect a :py:class:`Worker` when Celery starts."""
     from debusine.db.models import Worker
     from debusine.tasks import BaseTask
     from debusine.tasks.executors import analyze_worker_all_executors
     from debusine.tasks.models import WorkerType
     from debusine.worker.system_information import system_metadata
 
-    worker = Worker.objects.get_or_create_celery()
-    worker.concurrency = sender.concurrency
-    worker.save()
+    if sender.hostname.startswith("celery@"):
+        worker = Worker.objects.get_or_create_celery()
+        worker.concurrency = sender.concurrency
+        worker.save()
 
-    metadata = {
-        **system_metadata(WorkerType.CELERY),
-        **analyze_worker_all_executors(),
-        **BaseTask.analyze_worker_all_tasks(),
-    }
+        metadata = {
+            **system_metadata(WorkerType.CELERY),
+            **analyze_worker_all_executors(),
+            **BaseTask.analyze_worker_all_tasks(),
+        }
 
-    worker.set_dynamic_metadata(metadata)
-    worker.mark_connected()
+        worker.set_dynamic_metadata(metadata)
+        worker.mark_connected()
 
 
 # mypy complains that worker_shutdown.connect is untyped, which is true, but
 # we can't fix that here.
 @worker_shutdown.connect  # type: ignore[misc]
-def disconnect_worker(**kwargs: Any) -> None:
-    """Disconnect the appropriate :class:`Worker` when Celery exits."""
+def disconnect_worker(*, sender: WorkController, **kwargs: Any) -> None:
+    """Disconnect the appropriate :py:class:`Worker` when Celery exits."""
     from debusine.db.models import Worker
 
-    Worker.objects.get_or_create_celery().mark_disconnected()
+    if sender.hostname.startswith("celery@"):
+        Worker.objects.get_or_create_celery().mark_disconnected()
