@@ -11,10 +11,9 @@
 
 import inspect
 import logging
-import re
 from collections.abc import Callable
 from datetime import timedelta
-from typing import Any, ClassVar, TYPE_CHECKING
+from typing import Any, TYPE_CHECKING
 from unittest import mock
 
 from channels.db import database_sync_to_async
@@ -25,16 +24,15 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
     DebianSourcePackage,
+    TaskTypes,
 )
 from debusine.assets import KeyPurpose
-from debusine.db.context import context
 from debusine.db.models import (
-    Artifact,
+    CollectionItem,
     TaskDatabase,
     Token,
     WorkRequest,
     Worker,
-    Workspace,
     default_workspace,
 )
 from debusine.db.tests.utils import RunInParallelTransaction
@@ -45,19 +43,21 @@ from debusine.server.scheduler import (
     schedule_for_worker,
 )
 from debusine.server.tasks import ServerNoop
-from debusine.server.tasks.tests.helpers import TestBaseWaitTask
+from debusine.server.tasks.tests.helpers import SampleBaseWaitTask
 from debusine.server.tasks.wait.models import DelayData
 from debusine.server.workflows.models import WorkRequestWorkflowData
 from debusine.signing.tasks.models import GenerateKeyData
 from debusine.tasks import Lintian, Sbuild
 from debusine.tasks.models import (
+    ActionSkipIfLookupResultChanged,
     BaseDynamicTaskData,
     BaseTaskData,
+    EventReactions,
     LintianData,
     LintianInput,
-    LookupMultiple,
     LookupSingle,
-    TaskTypes,
+    OutputData,
+    OutputDataError,
     WorkerType,
 )
 from debusine.tasks.server import ArtifactInfo
@@ -117,7 +117,7 @@ class SchedulerTestCase(PlaygroundTestCase):
         # environment
         (
             "debian/match:codename=sid:architecture=amd64:format=tarball:"
-            "backend=unshare",
+            "backend=unshare:variant=",
             CollectionCategory.ENVIRONMENTS,
         ): ArtifactInfo(
             id=2,
@@ -147,7 +147,7 @@ class SchedulerTestCase(PlaygroundTestCase):
     ) -> Worker:
         """Create and return a Worker."""
         token = Token.objects.create(enabled=enabled)
-        worker = Worker.objects.create_with_fqdn('worker.lan', token)
+        worker = Worker.objects.create_with_fqdn('worker.lan', token=token)
 
         worker.dynamic_metadata = {
             "system:worker_type": worker_type,
@@ -188,171 +188,12 @@ class SchedulerTestCase(PlaygroundTestCase):
         self.worker.save()
 
 
-class TaskDatabaseTests(TestCase):
-    """Test database interaction in worker tasks."""
-
-    workspace: ClassVar[Workspace]
-    work_request: ClassVar[WorkRequest]
-
-    @classmethod
-    @context.disable_permission_checks()
-    def setUpTestData(cls) -> None:
-        """Set up common data for tests."""
-        super().setUpTestData()
-        cls.workspace = cls.playground.create_workspace(
-            name="test", public=True
-        )
-        cls.work_request = cls.playground.create_work_request(
-            task_name="noop", workspace=cls.workspace
-        )
-
-    @context.disable_permission_checks()
-    def test_lookup_single_artifact(self) -> None:
-        """Look up a single artifact."""
-        collection = self.playground.create_collection(
-            "debian", CollectionCategory.ENVIRONMENTS, workspace=self.workspace
-        )
-        artifact, _ = self.playground.create_artifact(
-            category=ArtifactCategory.SYSTEM_TARBALL,
-            data=create_system_tarball_data(
-                codename="bookworm", architecture="amd64"
-            ),
-        )
-        collection.manager.add_artifact(
-            artifact, user=self.playground.get_default_user()
-        )
-        task_db = TaskDatabase(self.work_request)
-
-        self.assertEqual(
-            task_db.lookup_single_artifact(
-                "debian@debian:environments/match:codename=bookworm"
-            ),
-            ArtifactInfo(
-                id=artifact.id, category=artifact.category, data=artifact.data
-            ),
-        )
-        self.assertEqual(
-            task_db.lookup_single_artifact(
-                "debian/match:codename=bookworm",
-                default_category=CollectionCategory.ENVIRONMENTS,
-            ),
-            ArtifactInfo(
-                id=artifact.id, category=artifact.category, data=artifact.data
-            ),
-        )
-        with self.assertRaisesRegex(
-            LookupError,
-            "'debian' does not specify a category and the context does not "
-            "supply a default",
-        ):
-            task_db.lookup_single_artifact("debian/match:codename=bookworm")
-
-    @context.disable_permission_checks()
-    def test_lookup_multiple_artifacts(self) -> None:
-        """Look up multiple artifacts."""
-        collection = self.playground.create_collection(
-            "debian", CollectionCategory.ENVIRONMENTS, workspace=self.workspace
-        )
-        artifacts: list[Artifact] = []
-        for codename in "bookworm", "trixie":
-            artifact, _ = self.playground.create_artifact(
-                category=ArtifactCategory.SYSTEM_TARBALL,
-                data=create_system_tarball_data(
-                    codename=codename, architecture="amd64"
-                ),
-            )
-            collection.manager.add_artifact(
-                artifact, user=self.playground.get_default_user()
-            )
-            artifacts.append(artifact)
-        task_db = TaskDatabase(self.work_request)
-
-        self.assertCountEqual(
-            task_db.lookup_multiple_artifacts(
-                LookupMultiple.parse_obj(
-                    {"collection": "debian@debian:environments"}
-                )
-            ),
-            [
-                ArtifactInfo(
-                    id=artifact.id,
-                    category=artifact.category,
-                    data=artifact.data,
-                )
-                for artifact in artifacts
-            ],
-        )
-        self.assertCountEqual(
-            task_db.lookup_multiple_artifacts(
-                LookupMultiple.parse_obj({"collection": "debian"}),
-                default_category=CollectionCategory.ENVIRONMENTS,
-            ),
-            [
-                ArtifactInfo(
-                    id=artifact.id,
-                    category=artifact.category,
-                    data=artifact.data,
-                )
-                for artifact in artifacts
-            ],
-        )
-        with self.assertRaisesRegex(
-            LookupError,
-            "'debian' does not specify a category and the context does not "
-            "supply a default",
-        ):
-            task_db.lookup_multiple_artifacts(
-                LookupMultiple.parse_obj({"collection": "debian"})
-            )
-
-    def test_lookup_single_collection(self) -> None:
-        """Look up a single collection."""
-        collection = self.playground.create_collection(
-            "debian", CollectionCategory.ENVIRONMENTS, workspace=self.workspace
-        )
-        task_db = TaskDatabase(self.work_request)
-
-        self.assertEqual(
-            task_db.lookup_single_collection("debian@debian:environments"),
-            collection.id,
-        )
-        self.assertEqual(
-            task_db.lookup_single_collection(
-                "debian",
-                default_category=CollectionCategory.ENVIRONMENTS,
-            ),
-            collection.id,
-        )
-        with self.assertRaisesRegex(
-            LookupError,
-            "'debian' does not specify a category and the context does not "
-            "supply a default",
-        ):
-            task_db.lookup_single_collection("debian")
-
-    def test_lookup_singleton_collection(self) -> None:
-        """Look up a singleton collection."""
-        collection = self.playground.create_singleton_collection(
-            CollectionCategory.PACKAGE_BUILD_LOGS, workspace=self.workspace
-        )
-        task_db = TaskDatabase(self.work_request)
-
-        self.assertEqual(
-            task_db.lookup_singleton_collection(
-                CollectionCategory.PACKAGE_BUILD_LOGS
-            ),
-            collection.id,
-        )
-        self.assertIsNone(
-            task_db.lookup_singleton_collection(CollectionCategory.TASK_HISTORY)
-        )
-
-
 class SchedulerTests(SchedulerTestCase, TestCase):
     """Test schedule() function."""
 
     def setUp(self) -> None:
         """Initialize test case."""
+        super().setUp()
         self.work_request_1 = self.create_sample_sbuild_work_request()
         self.work_request_2 = self.create_sample_sbuild_work_request()
         self.patch_task_database()
@@ -504,7 +345,7 @@ class SchedulerTests(SchedulerTestCase, TestCase):
     def test_wait_other_task(self) -> None:
         """schedule() leaves non-Delay Wait tasks running."""
 
-        class WaitNoop(TestBaseWaitTask[BaseTaskData, BaseDynamicTaskData]):
+        class WaitNoop(SampleBaseWaitTask[BaseTaskData, BaseDynamicTaskData]):
             TASK_VERSION = 1
 
             def _execute(self) -> bool:
@@ -532,6 +373,7 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
 
     def setUp(self) -> None:
         """Initialize test case."""
+        super().setUp()
         self.worker = self._create_worker(enabled=True)
         self.patch_task_database()
 
@@ -585,6 +427,48 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
 
         self.assertIsNone(result)
 
+    def test_skips_work_request_if_lookup_result_changed(self) -> None:
+        """Skip work request if an action requests it."""
+        collection = self.playground.create_collection(
+            "test", CollectionCategory.TEST
+        )
+        lookup = f"test@{CollectionCategory.TEST}/name:foo"
+        work_request = self.create_sample_sbuild_work_request()
+        work_request.event_reactions = EventReactions(
+            on_assignment=[
+                ActionSkipIfLookupResultChanged(
+                    lookup=lookup, collection_item_id=None
+                )
+            ]
+        )
+        work_request.save()
+        CollectionItem.objects.create_from_artifact(
+            self.playground.create_artifact()[0],
+            parent_collection=collection,
+            name="foo",
+            data={},
+            created_by_user=self.playground.get_default_user(),
+        )
+
+        with self.assertLogs(
+            "debusine.server.scheduler", level=logging.DEBUG
+        ) as log:
+            result = schedule_for_worker(self.worker)
+        work_request.refresh_from_db()
+
+        self.assertIn(
+            f"DEBUG:debusine.server.scheduler:"
+            f"Skipping work request {work_request} due to on_assignment action",
+            log.output,
+        )
+        self.assertIsNone(result)
+        self.assertEqual(work_request.status, WorkRequest.Statuses.COMPLETED)
+        self.assertEqual(work_request.result, WorkRequest.Results.SKIPPED)
+        self.assertEqual(
+            work_request.output_data,
+            OutputData(skip_reason=f"Result of lookup {lookup!r} changed"),
+        )
+
     def test_with_pending_unassigned_work_request(self) -> None:
         """schedule_for_worker() picks up the pending work request."""
         with override_settings(DISABLE_AUTOMATIC_SCHEDULING=True):
@@ -613,14 +497,31 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
             result = schedule_for_worker(self.worker)
         work_request.refresh_from_db()
 
+        expected_message = (
+            "Failed to configure: 3 validation errors for SbuildData"
+        )
         self.assertIn(
-            f"WorkRequest {work_request.id} failed to configure, aborting it. "
-            f"Task data: {work_request.task_data} Error: 3 validation errors",
+            f"ERROR:debusine.server.scheduler:"
+            f"Error assigning work request "
+            f"{work_request.task_type}/{work_request.task_name} "
+            f"({work_request.id}): {expected_message}",
             "\n".join(log.output),
         )
         self.assertIsNone(result)
         self.assertEqual(work_request.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(work_request.result, WorkRequest.Results.ERROR)
+        assert work_request.output_data is not None
+        self.assertIsInstance(work_request.output_data, OutputData)
+        assert work_request.output_data.errors is not None
+        self.assertEqual(len(work_request.output_data.errors), 1)
+        self.assertTrue(
+            work_request.output_data.errors[0].message.startswith(
+                expected_message
+            )
+        )
+        self.assertEqual(
+            work_request.output_data.errors[0].code, "configure-failed"
+        )
 
     def test_with_failing_compute_dynamic_data(self) -> None:
         """schedule_for_worker() handles failure to compute dynamic data."""
@@ -636,21 +537,33 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
             result = schedule_for_worker(self.worker)
         work_request.refresh_from_db()
 
-        lines = re.split(r"(?= [A-Z])", log.output[0])
+        expected_message = (
+            f"Failed to compute dynamic data: "
+            f"('bad-lookup', {CollectionCategory.ENVIRONMENTS!r})"
+        )
         self.assertEqual(
-            lines,
+            log.output,
             [
-                "WARNING:debusine.server.scheduler:"
-                f"WorkRequest {work_request.id}"
-                " failed to pre-process task data, aborting it.",
-                f" Task data: {work_request.task_data}",
-                f" Error: ('bad-lookup', {CollectionCategory.ENVIRONMENTS!r})",
+                f"ERROR:debusine.server.scheduler:"
+                f"Error assigning work request "
+                f"{work_request.task_type}/{work_request.task_name} "
+                f"({work_request.id}): {expected_message}"
             ],
         )
 
         self.assertIsNone(result)
         self.assertEqual(work_request.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(work_request.result, WorkRequest.Results.ERROR)
+        self.assertEqual(
+            work_request.output_data,
+            OutputData(
+                errors=[
+                    OutputDataError(
+                        message=expected_message, code="dynamic-data-failed"
+                    )
+                ]
+            ),
+        )
 
     @override_settings(DISABLE_AUTOMATIC_SCHEDULING=True)
     def test_computes_dynamic_data_with_worker_host_architecture(self) -> None:
@@ -684,7 +597,7 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
             ),
             (
                 "debian/match:codename=bookworm:architecture=i386:"
-                "format=tarball:backend=unshare",
+                "format=tarball:backend=unshare:variant=",
                 CollectionCategory.ENVIRONMENTS,
             ): ArtifactInfo(
                 id=2,
@@ -873,10 +786,9 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
     )
     async def test_schedule_from_work_request_changed(self) -> None:
         """End to end: from WorkRequest.save(), notification to schedule()."""
-        with context.disable_permission_checks():
-            work_request_1 = await database_sync_to_async(
-                self.create_sample_sbuild_work_request
-            )()
+        work_request_1 = await database_sync_to_async(
+            self.create_sample_sbuild_work_request
+        )()
 
         self.assertIsNone(work_request_1.worker)
 
@@ -929,10 +841,9 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
         """End to end: from Worker.save() to schedule_for_worker()."""
         await database_sync_to_async(self.worker.mark_disconnected)()
 
-        with context.disable_permission_checks():
-            work_request_1 = await database_sync_to_async(
-                self.create_sample_sbuild_work_request
-            )()
+        work_request_1 = await database_sync_to_async(
+            self.create_sample_sbuild_work_request
+        )()
 
         # The Worker is not connected: the work_request_1.worker is None
         self.assertIsNone(work_request_1.worker)
@@ -952,8 +863,7 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
     @override_settings(DISABLE_AUTOMATIC_SCHEDULING=True)
     def test_when_worker_is_locked(self) -> None:
         """schedule_for_worker() fails when it fails to lock the Worker."""
-        with context.disable_permission_checks():
-            self.create_sample_sbuild_work_request()
+        self.create_sample_sbuild_work_request()
 
         thread = RunInParallelTransaction(
             lambda: Worker.objects.select_for_update().get(id=self.worker.id)
@@ -968,8 +878,7 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
     @override_settings(DISABLE_AUTOMATIC_SCHEDULING=True)
     def test_when_work_request_is_locked(self) -> None:
         """schedule_for_worker() fails when it fails to lock the WorkRequest."""
-        with context.disable_permission_checks():
-            work_request = self.create_sample_sbuild_work_request()
+        work_request = self.create_sample_sbuild_work_request()
 
         thread = RunInParallelTransaction(
             lambda: WorkRequest.objects.select_for_update().get(
@@ -1003,15 +912,14 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
     @override_settings(DISABLE_AUTOMATIC_SCHEDULING=True)
     def test_server_task(self) -> None:
         """schedule_for_worker() dispatches server tasks to Celery."""
-        with context.disable_permission_checks():
-            worker = Worker.objects.get_or_create_celery()
-            worker.dynamic_metadata = {
-                "system:worker_type": WorkerType.CELERY,
-                "server:servernoop:version": ServerNoop.TASK_VERSION,
-            }
-            work_request = self.playground.create_work_request(
-                task_type=TaskTypes.SERVER, task_name="servernoop"
-            )
+        worker = Worker.objects.get_or_create_celery()
+        worker.dynamic_metadata = {
+            "system:worker_type": WorkerType.CELERY,
+            "server:servernoop:version": ServerNoop.TASK_VERSION,
+        }
+        work_request = self.playground.create_work_request(
+            task_type=TaskTypes.SERVER, task_name="servernoop"
+        )
 
         with mock.patch(
             "debusine.server.scheduler.run_server_task.apply_async"
@@ -1026,14 +934,11 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
     def test_workflow_callbacks(self) -> None:
         """schedule() dispatches workflow callbacks to Celery."""
         WorkRequest.objects.all().delete()
-        with context.disable_permission_checks():
-            parent = self.playground.create_workflow()
+        parent = self.playground.create_workflow()
         parent.mark_running()
         wr = WorkRequest.objects.create_workflow_callback(
             parent=parent, step="test"
         )
-        wr.status = WorkRequest.Statuses.PENDING
-        wr.save()
 
         with mock.patch(
             "debusine.server.scheduler.run_workflow_task.apply_async"
@@ -1046,11 +951,117 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
         wr.refresh_from_db()
         self.assertEqual(wr.status, WorkRequest.Statuses.RUNNING)
 
+    def test_workflow_callback_another_workflow_running(self) -> None:
+        """Callbacks for multiple workflows may run concurrently."""
+        WorkRequest.objects.all().delete()
+        workflows = [self.playground.create_workflow() for _ in range(3)]
+        for workflow in workflows:
+            workflow.mark_running()
+        wrs = [
+            WorkRequest.objects.create_workflow_callback(
+                parent=workflow, step="test"
+            )
+            for workflow in workflows[:2]
+        ]
+
+        with mock.patch(
+            "debusine.server.scheduler.run_workflow_task.apply_async"
+        ) as mock_apply_async:
+            self.assertEqual(schedule(), wrs)
+
+        self.assertEqual(mock_apply_async.call_count, 2)
+        mock_apply_async.assert_has_calls(
+            [
+                mock.call(args=(wrs[0].id,), task_id=mock.ANY),
+                mock.call(args=(wrs[1].id,), task_id=mock.ANY),
+            ]
+        )
+        self.assertQuerySetEqual(
+            WorkRequest.objects.filter(id__in={wr.id for wr in wrs})
+            .order_by("id")
+            .values_list("status", flat=True),
+            [WorkRequest.Statuses.RUNNING] * len(wrs),
+        )
+
+        wrs.append(
+            WorkRequest.objects.create_workflow_callback(
+                parent=workflows[-1], step="test"
+            )
+        )
+        # A running work request in the same parent workflow that isn't a
+        # workflow callback doesn't stop the workflow callback running.
+        self.playground.create_work_request(
+            parent=workflows[-1], assign_new_worker=True, mark_running=True
+        )
+
+        with mock.patch(
+            "debusine.server.scheduler.run_workflow_task.apply_async"
+        ) as mock_apply_async:
+            self.assertCountEqual(schedule(), [wrs[-1]])
+        mock_apply_async.assert_called_once_with(
+            args=(wrs[-1].id,), task_id=mock.ANY
+        )
+        self.assertQuerySetEqual(
+            WorkRequest.objects.filter(id__in={wr.id for wr in wrs})
+            .order_by("id")
+            .values_list("status", flat=True),
+            [WorkRequest.Statuses.RUNNING] * len(wrs),
+        )
+
+    def test_workflow_callback_same_workflow_running(self) -> None:
+        """Callbacks for the same workflow may not run concurrently."""
+        WorkRequest.objects.all().delete()
+        workflow = self.playground.create_workflow()
+        workflow.mark_running()
+        wrs = [
+            WorkRequest.objects.create_workflow_callback(
+                parent=workflow, step=f"test{i}"
+            )
+            for i in range(2)
+        ]
+
+        with mock.patch(
+            "debusine.server.scheduler.run_workflow_task.apply_async"
+        ) as mock_apply_async:
+            self.assertEqual(schedule(), [wrs[0]])
+
+        mock_apply_async.assert_called_once_with(
+            args=(wrs[0].id,), task_id=mock.ANY
+        )
+        self.assertQuerySetEqual(
+            WorkRequest.objects.filter(id__in={wr.id for wr in wrs})
+            .order_by("id")
+            .values_list("status", flat=True),
+            [WorkRequest.Statuses.RUNNING, WorkRequest.Statuses.PENDING],
+        )
+
+        wrs.append(
+            WorkRequest.objects.create_workflow_callback(
+                parent=workflow, step="test2"
+            )
+        )
+
+        with mock.patch(
+            "debusine.server.scheduler.run_workflow_task.apply_async"
+        ) as mock_apply_async:
+            self.assertEqual(schedule(), [])
+
+        mock_apply_async.assert_not_called()
+        self.assertQuerySetEqual(
+            WorkRequest.objects.filter(id__in={wr.id for wr in wrs})
+            .order_by("id")
+            .values_list("status", flat=True),
+            [
+                WorkRequest.Statuses.RUNNING,
+                WorkRequest.Statuses.PENDING,
+                WorkRequest.Statuses.PENDING,
+            ],
+        )
+
     def test_workflows(self) -> None:
         """schedule() dispatches workflows to Celery."""
         WorkRequest.objects.all().delete()
-        with context.disable_permission_checks():
-            wr = self.playground.create_workflow(task_name="noop")
+        wr = self.playground.create_workflow(task_name="noop")
 
         with mock.patch(
             "debusine.server.scheduler.run_workflow_task.apply_async"
@@ -1065,13 +1076,11 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
 
     def test_sub_workflow(self) -> None:
         """schedule() orchestrates pending sub-workflows via their root."""
-        with context.disable_permission_checks():
-            root_wr = self.playground.create_workflow(task_name="noop")
-            sub_wr = self.playground.create_workflow(parent=root_wr)
-            # Mark the root workflow running, to simulate the situation
-            # where it can only do part of its work and has to be called
-            # back later.
-            root_wr.mark_running()
+        root_wr = self.playground.create_workflow(task_name="noop")
+        sub_wr = self.playground.create_workflow(parent=root_wr)
+        # Mark the root workflow running, to simulate the situation where it
+        # can only do part of its work and has to be called back later.
+        root_wr.mark_running()
 
         with mock.patch(
             "debusine.server.scheduler.run_workflow_task.apply_async",
@@ -1088,9 +1097,8 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
 
     def test_groups_workflows_by_root(self) -> None:
         """schedule() groups pending workflows by their root workflow."""
-        with context.disable_permission_checks():
-            root_wr = self.playground.create_workflow(task_name="noop")
-            sub_wr = self.playground.create_workflow(parent=root_wr)
+        root_wr = self.playground.create_workflow(task_name="noop")
+        sub_wr = self.playground.create_workflow(parent=root_wr)
 
         with mock.patch(
             "debusine.server.scheduler.run_workflow_task.apply_async"
