@@ -13,7 +13,6 @@ from functools import cached_property
 
 from debian.debian_support import DpkgArchTable
 
-from debusine.artifacts import SourcePackage
 from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
@@ -24,6 +23,7 @@ from debusine.artifacts.models import (
 from debusine.client.models import LookupChildType
 from debusine.db.models import Artifact, TaskDatabase, WorkRequest
 from debusine.server.collections.lookup import lookup_single
+from debusine.server.workflows import workflow_utils
 from debusine.server.workflows.base import Workflow, WorkflowValidationError
 from debusine.server.workflows.models import (
     SbuildWorkflowData,
@@ -37,6 +37,7 @@ from debusine.tasks.models import (
     BackendType,
     BaseDynamicTaskData,
     SbuildBuildComponent,
+    SbuildBuildDepResolver,
     SbuildData,
 )
 from debusine.tasks.server import TaskDatabaseInterface
@@ -53,30 +54,15 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
         if self.data.backend == BackendType.AUTO:
             self.data.backend = BackendType.UNSHARE
 
-    @cached_property
-    def source_package(self) -> Artifact:
-        """Source package artifact."""
-        artifact = lookup_single(
-            self.data.input.source_artifact,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.get_workflow_root(),
-            expect_type=LookupChildType.ARTIFACT,
-        ).artifact
-        return self.locate_debian_source_package(
-            "input.source_artifact", artifact
-        )
-
-    @cached_property
-    def source_package_data(self) -> DebianSourcePackage:
-        """Parsed source package artifact data."""
-        return SourcePackage.create_data(self.source_package.data)
-
-    def compute_dynamic_data(
+    def build_dynamic_data(
         self, task_database: TaskDatabaseInterface  # noqa: U100
     ) -> BaseDynamicTaskData:
-        """Compute dynamic data for this workflow."""
-        source_data = self.source_package_data
+        """
+        Compute dynamic data for this workflow.
+
+        :subject: package name of ``input.source_artifact``
+        """
+        source_data = workflow_utils.source_package_data(self)
         return BaseDynamicTaskData(
             subject=source_data.name,
             parameter_summary=f"{source_data.name}_{source_data.version}",
@@ -86,7 +72,10 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
     def binary_names(self) -> list[str]:
         """Binary names that may be built."""
         return re.split(
-            r"\s*,\s*", self.source_package_data.dsc_fields.get("Binary", "")
+            r"\s*,\s*",
+            workflow_utils.source_package_data(self).dsc_fields.get(
+                "Binary", ""
+            ),
         )
 
     @cached_property
@@ -95,7 +84,9 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
         workflow_arches = set(self.data.architectures)
 
         package_arches = set(
-            self.source_package_data.dsc_fields.get("Architecture", "").split()
+            workflow_utils.source_package_data(self)
+            .dsc_fields.get("Architecture", "")
+            .split()
         )
         if not package_arches:
             raise WorkflowValidationError("package architecture list is empty")
@@ -181,6 +172,7 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
                     else self.data.backend
                 ),
                 default_category=CollectionCategory.ENVIRONMENTS,
+                try_variant="sbuild",
             ).id
         except KeyError:
             raise WorkflowValidationError(
@@ -242,8 +234,10 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
         host_architecture = self._get_host_architecture(architecture)
         input_ = self.data.input.copy()
 
-        input_.source_artifact = self.locate_debian_source_package_lookup(
-            "input.source_artifact", self.data.input.source_artifact
+        input_.source_artifact = (
+            workflow_utils.locate_debian_source_package_lookup(
+                self, "input.source_artifact", self.data.input.source_artifact
+            )
         )
 
         source_artifact = lookup_single(
@@ -256,6 +250,29 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
         assert source_artifact.category == ArtifactCategory.SOURCE_PACKAGE
         debian_source_package = DebianSourcePackage(**source_artifact.data)
 
+        vendor, codename = self.data.target_distribution.split(":", 1)
+        extra_repositories = workflow_utils.configure_for_overlay_suite(
+            self,
+            extra_repositories=self.data.extra_repositories,
+            vendor=vendor,
+            codename=codename,
+            environment=environment,
+            backend=self.data.backend,
+            architecture=host_architecture,
+            try_variant="sbuild",
+        )
+
+        build_dep_resolver: SbuildBuildDepResolver | None = None
+        aspcud_criteria: str | None = None
+        if (vendor, codename) == ("debian", "experimental"):
+            # https://salsa.debian.org/dsa-team/mirror/dsa-puppet/-/blob/production/modules/buildd/templates/sbuild.conf.erb
+            # TODO: Implement this entirely in task-configuration
+            build_dep_resolver = SbuildBuildDepResolver.ASPCUD
+            aspcud_criteria = (
+                "-count(down),-count(changed,APT-Release:=/experimental/),"
+                "-removed,-changed,-new"
+            )
+
         task_data = SbuildData(
             input=input_,
             host_architecture=host_architecture,
@@ -269,8 +286,10 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
                 )
             ],
             build_profiles=self.data.build_profiles,
-            extra_repositories=self.data.extra_repositories,
+            extra_repositories=extra_repositories,
             binnmu=self.data.binnmu,
+            build_dep_resolver=build_dep_resolver,
+            aspcud_criteria=aspcud_criteria,
         )
         wr = self.work_request_ensure_child(
             task_name="sbuild",
@@ -302,15 +321,15 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
             build_logs_collection_id = None
 
         if build_logs_collection_id is not None:
-            vendor, codename = self.data.target_distribution.split(":", 1)
-
             variables = {
                 "work_request_id": wr.id,
                 "vendor": vendor,
                 "codename": codename,
                 "architecture": host_architecture,
-                "srcpkg_name": self.source_package_data.name,
-                "srcpkg_version": self.source_package_data.version,
+                "srcpkg_name": workflow_utils.source_package_data(self).name,
+                "srcpkg_version": workflow_utils.source_package_data(
+                    self
+                ).version,
             }
             wr.add_event_reaction(
                 "on_creation",
@@ -345,7 +364,10 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
                 ArtifactCategory.BINARY_PACKAGE,
                 f"{self.data.prefix}signing-template-{architecture}-"
                 f"{binary_package_name}",
-                data={"architecture": architecture},
+                data={
+                    "architecture": architecture,
+                    "binary_package_name": binary_package_name,
+                },
                 artifact_filters={
                     "data__deb_fields__Package": binary_package_name
                 },
