@@ -12,10 +12,13 @@ from typing import Any, ClassVar
 
 from django.test import override_settings
 
-from debusine.artifacts.models import ArtifactCategory, CollectionCategory
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    CollectionCategory,
+    TaskTypes,
+)
 from debusine.assets import KeyPurpose
 from debusine.client.models import LookupChildType
-from debusine.db.context import context
 from debusine.db.models import Artifact, TaskDatabase, WorkRequest
 from debusine.server.scheduler import schedule
 from debusine.server.workflows import (
@@ -33,7 +36,7 @@ from debusine.tasks.models import (
     BackendType,
     BaseDynamicTaskData,
     CollectionItemMatcherKind,
-    TaskTypes,
+    LintianFailOnSeverity,
 )
 from debusine.test.django import TestCase
 
@@ -44,7 +47,6 @@ class DebianPipelineWorkflowTests(TestCase):
     source_artifact: ClassVar[Artifact]
 
     @classmethod
-    @context.disable_permission_checks()
     def setUpTestData(cls) -> None:
         """Set up common data."""
         super().setUpTestData()
@@ -99,14 +101,12 @@ class DebianPipelineWorkflowTests(TestCase):
 
         workflow.validate_input()
 
-    def test_validate_input_bad_suite_collection(self) -> None:
+    def test_validate_input_bad_qa_suite(self) -> None:
         """validate_input raises errors in looking up a suite."""
         workflow = self.create_debian_pipeline_workflow(
             extra_task_data={
+                "qa_suite": "nonexistent@debian:suite",
                 "enable_reverse_dependencies_autopkgtest": True,
-                "reverse_dependencies_autopkgtest_suite": (
-                    "nonexistent@debian:suite"
-                ),
             },
             validate=False,
         )
@@ -182,20 +182,19 @@ class DebianPipelineWorkflowTests(TestCase):
         The user didn't specify "architectures", DebianPipelineWorkflow
         checks available architectures and "all" and use them.
         """
-        with context.disable_permission_checks():
-            collection = self.playground.create_collection(
-                "debian",
-                CollectionCategory.ENVIRONMENTS,
-                workspace=self.playground.get_default_workspace(),
+        collection = self.playground.create_collection(
+            "debian",
+            CollectionCategory.ENVIRONMENTS,
+            workspace=self.playground.get_default_workspace(),
+        )
+        for arch in ["amd64", "i386"]:
+            artifact, _ = self.playground.create_artifact(
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data={"codename": "bookworm", "architecture": arch},
+            )
+            collection.manager.add_artifact(
+                artifact, user=self.playground.get_default_user()
             )
-            for arch in ["amd64", "i386"]:
-                artifact, _ = self.playground.create_artifact(
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data={"codename": "bookworm", "architecture": arch},
-                )
-                collection.manager.add_artifact(
-                    artifact, user=self.playground.get_default_user()
-                )
 
         workflow = self.orchestrate(
             extra_data={
@@ -358,10 +357,11 @@ class DebianPipelineWorkflowTests(TestCase):
                 "enable_lintian": True,
                 "enable_piuparts": False,
                 "enable_reverse_dependencies_autopkgtest": False,
+                "enable_debdiff": False,
                 "lintian_backend": BackendType.AUTO,
-                "lintian_fail_on_severity": "none",
+                "lintian_fail_on_severity": LintianFailOnSeverity.ERROR,
                 "piuparts_backend": BackendType.AUTO,
-                "reverse_dependencies_autopkgtest_suite": None,
+                "qa_suite": None,
                 "source_artifact": f"{self.source_artifact.id}@artifacts",
                 "vendor": "debian",
             },
@@ -398,10 +398,11 @@ class DebianPipelineWorkflowTests(TestCase):
                 "enable_lintian": True,
                 "enable_piuparts": False,
                 "enable_reverse_dependencies_autopkgtest": False,
+                "enable_debdiff": False,
                 "lintian_backend": BackendType.AUTO,
-                "lintian_fail_on_severity": "none",
+                "lintian_fail_on_severity": LintianFailOnSeverity.ERROR,
                 "piuparts_backend": BackendType.AUTO,
-                "reverse_dependencies_autopkgtest_suite": None,
+                "qa_suite": None,
                 "source_artifact": self.source_artifact.id,
                 "vendor": "debian",
             },
@@ -439,10 +440,11 @@ class DebianPipelineWorkflowTests(TestCase):
                 "enable_lintian": False,
                 "enable_piuparts": False,
                 "enable_reverse_dependencies_autopkgtest": False,
+                "enable_debdiff": False,
                 "lintian_backend": BackendType.AUTO,
-                "lintian_fail_on_severity": "none",
+                "lintian_fail_on_severity": LintianFailOnSeverity.ERROR,
                 "piuparts_backend": BackendType.AUTO,
-                "reverse_dependencies_autopkgtest_suite": None,
+                "qa_suite": None,
                 "source_artifact": self.source_artifact.id,
                 "vendor": "debian",
             },
@@ -480,15 +482,75 @@ class DebianPipelineWorkflowTests(TestCase):
                 "enable_lintian": False,
                 "enable_piuparts": True,
                 "enable_reverse_dependencies_autopkgtest": False,
+                "enable_debdiff": False,
                 "lintian_backend": BackendType.AUTO,
-                "lintian_fail_on_severity": "none",
+                "lintian_fail_on_severity": LintianFailOnSeverity.ERROR,
                 "piuparts_backend": BackendType.AUTO,
-                "reverse_dependencies_autopkgtest_suite": None,
+                "qa_suite": None,
                 "source_artifact": self.source_artifact.id,
                 "vendor": "debian",
             },
         )
 
+    def test_populate_qa_debdiff(self) -> None:
+        """Test populate create qa: debdiff enabled."""
+        trixie = self.playground.create_collection(
+            name="trixie",
+            category=CollectionCategory.SUITE,
+        )
+
+        workflow = self.orchestrate(
+            extra_data={
+                "architectures": ["amd64"],
+                "source_artifact": self.source_artifact.id,
+                "enable_debdiff": True,
+                "qa_suite": "trixie@debian:suite",
+            }
+        )
+
+        qa = workflow.children.get(task_name="qa", task_type=TaskTypes.WORKFLOW)
+
+        self.assertEqual(
+            qa.task_data,
+            {
+                "arch_all_host_architecture": "amd64",
+                "architectures": ["amd64"],
+                "autopkgtest_backend": BackendType.AUTO,
+                "binary_artifacts": [
+                    {
+                        "collection": "internal@collections",
+                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
+                        "name_matcher": {
+                            "kind": CollectionItemMatcherKind.STARTSWITH,
+                            "value": "build-",
+                        },
+                    }
+                ],
+                "codename": "bookworm",
+                "extra_repositories": None,
+                "enable_autopkgtest": False,
+                "enable_lintian": False,
+                "enable_piuparts": False,
+                "enable_reverse_dependencies_autopkgtest": False,
+                "enable_debdiff": True,
+                "lintian_backend": BackendType.AUTO,
+                "lintian_fail_on_severity": LintianFailOnSeverity.ERROR,
+                "piuparts_backend": BackendType.AUTO,
+                "qa_suite": str(trixie),
+                "source_artifact": self.source_artifact.id,
+                "vendor": "debian",
+            },
+        )
+
+        self.assertEqual(qa.status, WorkRequest.Statuses.RUNNING)
+
+        self.assertEqual(
+            qa.children.filter(
+                task_name="debdiff", task_type=TaskTypes.WORKFLOW
+            ).count(),
+            1,
+        )
+
     def test_populate_qa_piuparts_configuration(self) -> None:
         """Test populate create qa: piuparts non-default configuration."""
         workflow = self.orchestrate(
@@ -523,11 +585,12 @@ class DebianPipelineWorkflowTests(TestCase):
                 "enable_lintian": False,
                 "enable_piuparts": True,
                 "enable_reverse_dependencies_autopkgtest": False,
+                "enable_debdiff": False,
                 "lintian_backend": BackendType.AUTO,
-                "lintian_fail_on_severity": "none",
+                "lintian_fail_on_severity": LintianFailOnSeverity.ERROR,
                 "piuparts_backend": BackendType.INCUS_LXC,
                 "piuparts_environment": "debian/match:codename=trixie",
-                "reverse_dependencies_autopkgtest_suite": None,
+                "qa_suite": None,
                 "source_artifact": self.source_artifact.id,
                 "vendor": "debian",
             },
@@ -715,6 +778,7 @@ class DebianPipelineWorkflowTests(TestCase):
         self.assertEqual(
             package_upload.task_data,
             {
+                "arch_all_host_architecture": "amd64",
                 "binary_artifacts": [
                     {
                         "collection": "internal@collections",
@@ -728,6 +792,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 "codename": "trixie",
                 "delayed_days": None,
                 "key": None,
+                "binary_key": None,
                 "merge_uploads": False,
                 "require_signature": True,
                 "since_version": "1.1",
@@ -848,6 +913,7 @@ class DebianPipelineWorkflowTests(TestCase):
         self.assertEqual(
             package_upload_source.task_data,
             {
+                "arch_all_host_architecture": "amd64",
                 "binary_artifacts": [
                     {
                         "collection": "internal@collections",
@@ -861,6 +927,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 "codename": "trixie",
                 "delayed_days": None,
                 "key": None,
+                "binary_key": None,
                 "merge_uploads": False,
                 "require_signature": True,
                 "since_version": "1.1",
@@ -935,6 +1002,7 @@ class DebianPipelineWorkflowTests(TestCase):
         self.assertEqual(
             package_upload_signed_amd64.task_data,
             {
+                "arch_all_host_architecture": "amd64",
                 "binary_artifacts": [
                     {
                         "collection": "internal@collections",
@@ -948,6 +1016,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 "codename": "trixie",
                 "delayed_days": None,
                 "key": None,
+                "binary_key": None,
                 "merge_uploads": False,
                 "require_signature": True,
                 "since_version": "1.1",
@@ -1091,10 +1160,12 @@ class DebianPipelineWorkflowTests(TestCase):
         self.assertEqual(
             package_upload.task_data,
             {
+                "arch_all_host_architecture": "amd64",
                 "binary_artifacts": [],
                 "codename": "trixie",
                 "delayed_days": None,
                 "key": None,
+                "binary_key": None,
                 "merge_uploads": False,
                 "require_signature": True,
                 "since_version": "1.1",
@@ -1134,6 +1205,7 @@ class DebianPipelineWorkflowTests(TestCase):
         self.assertEqual(
             package_upload.task_data,
             {
+                "arch_all_host_architecture": "amd64",
                 "binary_artifacts": [
                     {
                         "collection": "internal@collections",
@@ -1147,6 +1219,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 "codename": "trixie",
                 "delayed_days": 3,
                 "key": None,
+                "binary_key": None,
                 "merge_uploads": False,
                 "require_signature": True,
                 "since_version": "1.1",
