@@ -34,9 +34,12 @@ except ImportError:
     import pydantic as pydantic  # type: ignore
 
 from debusine.artifacts import WorkRequestDebugLogs
-from debusine.artifacts.models import ArtifactCategory, EmptyArtifactData
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    EmptyArtifactData,
+    TaskTypes,
+)
 from debusine.client.debusine import Debusine
-from debusine.client.models import ArtifactResponse
 from debusine.tasks import (
     BaseExternalTask,
     BaseTask,
@@ -59,67 +62,59 @@ from debusine.tasks.models import (
     BaseTaskDataWithExecutor,
     BaseTaskDataWithExtraRepositories,
     ExtraRepository,
-    TaskTypes,
     WorkerType,
 )
 from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
     FakeTaskDatabase,
-    TestBaseExternalTask,
-    TestBaseTask,
-    TestBaseTaskWithExecutor,
+    SampleBaseExternalTask,
+    SampleBaseTask,
+    SampleBaseTaskWithExecutor,
 )
 from debusine.test import TestCase
 from debusine.test.test_utils import (
+    create_artifact_response,
     create_system_tarball_data,
     preserve_task_registry,
 )
 
 
-class TestBaseTask1(
-    TestBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
+class SampleBaseTask1(
+    SampleBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
 ):
     """Sample class to test BaseExternalTask class."""
 
-    __test__ = False
-
     def run(self, execute_directory: Path) -> bool:
         """Unused abstract method from BaseExternalTask."""
         raise NotImplementedError()
 
 
-class TestBaseTaskWithExecutor1(
-    TestBaseTaskWithExecutor[
+class SampleBaseTaskWithExecutor1(
+    SampleBaseTaskWithExecutor[
         BaseTaskDataWithExecutor, BaseDynamicTaskDataWithExecutor
     ],
 ):
     """Sample class to test BaseTaskWithExecutor class."""
 
-    __test__ = False
-
     def run(self, execute_directory: Path) -> bool:
         """Unused abstract method from BaseExternalTask."""
         raise NotImplementedError()
 
 
-class TestBaseTask2Data(BaseTaskData):
-    """Data representation for TestBaseTask2."""
-
-    __test__ = False
+class SampleBaseTask2Data(BaseTaskData):
+    """Data representation for SampleBaseTask2."""
 
     foo: str
     bar: float | None = None
     host_architecture: str | None = None
 
 
-class TestBaseTask2(
-    TestBaseExternalTask[TestBaseTask2Data, BaseDynamicTaskData],
+class SampleBaseTask2(
+    SampleBaseExternalTask[SampleBaseTask2Data, BaseDynamicTaskData],
 ):
     """Test BaseExternalTask class with data validation."""
 
-    __test__ = False
-
     TASK_VERSION = 1
 
     def run(self, execute_directory: Path) -> bool:
@@ -127,24 +122,20 @@ class TestBaseTask2(
         raise NotImplementedError()
 
 
-class TestBaseTaskDataWithExecutorAndArchitecture(BaseTaskDataWithExecutor):
+class SampleBaseTaskDataWithExecutorAndArchitecture(BaseTaskDataWithExecutor):
     """BaseTaskDataWithExecutor with a host architecture."""
 
-    __test__ = False
-
     host_architecture: str | None = None
 
 
-class TestBaseTaskWithExecutorAndArchitecture(
-    TestBaseTaskWithExecutor[
-        TestBaseTaskDataWithExecutorAndArchitecture,
+class SampleBaseTaskWithExecutorAndArchitecture(
+    SampleBaseTaskWithExecutor[
+        SampleBaseTaskDataWithExecutorAndArchitecture,
         BaseDynamicTaskDataWithExecutor,
     ],
 ):
     """Test BaseTaskWithExecutor with a host architecture."""
 
-    __test__ = False
-
     TASK_VERSION = 1
 
     def run(self, execute_directory: Path) -> bool:
@@ -152,8 +143,8 @@ class TestBaseTaskWithExecutorAndArchitecture(
         raise NotImplementedError()
 
 
-class TestExtraRepositoryMixin(
-    TestBaseTask[BaseTaskDataWithExtraRepositories, BaseDynamicTaskData],
+class SampleExtraRepositoryMixin(
+    SampleBaseTask[BaseTaskDataWithExtraRepositories, BaseDynamicTaskData],
     ExtraRepositoryMixin[
         BaseTaskDataWithExtraRepositories, BaseDynamicTaskData
     ],
@@ -165,9 +156,9 @@ class TestExtraRepositoryMixin(
         raise NotImplementedError()
 
 
-class TestDefaultDynamicData(
+class SampleDefaultDynamicData(
     DefaultDynamicData[BaseTaskData],
-    TestBaseTask[BaseTaskData, BaseDynamicTaskData],
+    SampleBaseTask[BaseTaskData, BaseDynamicTaskData],
 ):
     """Test DefaultDynamicData."""
 
@@ -191,13 +182,15 @@ class BaseTaskTests(TestCase):
 
     def setUp(self) -> None:
         """Create the shared attributes."""
-        self.task = TestBaseTask1({})
-        self.task2 = TestBaseTask2({"foo": "bar"})
+        super().setUp()
+        self.task = SampleBaseTask1({})
+        self.task2 = SampleBaseTask2({"foo": "bar"})
 
     def tearDown(self) -> None:
         """Delete directory to avoid ResourceWarning."""
         if self.task._debug_log_files_directory:
             self.task._debug_log_files_directory.cleanup()
+        super().tearDown()
 
     def test_task_initialize_member_variables(self) -> None:
         """Member variables expected used in other methods are initialized."""
@@ -206,7 +199,7 @@ class BaseTaskTests(TestCase):
 
     def test_name_is_set(self) -> None:
         """task.name is built from class name."""
-        self.assertEqual(self.task.name, "testbasetask1")
+        self.assertEqual(self.task.name, "samplebasetask1")
 
     def test_logger_is_configured(self) -> None:
         """task.logger is available."""
@@ -223,7 +216,7 @@ class BaseTaskTests(TestCase):
         # Cannot import ServerNoop or NoopWorkflow because sometimes these
         # tests are run without the django test runner
 
-        class TestServer(BaseTask[BaseTaskData, BaseDynamicTaskData]):
+        class SampleServer(BaseTask[BaseTaskData, BaseDynamicTaskData]):
             TASK_TYPE = TaskTypes.SERVER
 
             def _execute(self) -> bool:
@@ -233,10 +226,10 @@ class BaseTaskTests(TestCase):
                 raise NotImplementedError()
 
         self.assertEqual(
-            TestServer.prefix_with_task_name("foo"), "server:testserver:foo"
+            SampleServer.prefix_with_task_name("foo"), "server:sampleserver:foo"
         )
 
-        class TestInternal(BaseTask[BaseTaskData, BaseDynamicTaskData]):
+        class SampleInternal(BaseTask[BaseTaskData, BaseDynamicTaskData]):
             TASK_TYPE = TaskTypes.INTERNAL
 
             def _execute(self) -> bool:
@@ -246,11 +239,11 @@ class BaseTaskTests(TestCase):
                 raise NotImplementedError()
 
         self.assertEqual(
-            TestInternal.prefix_with_task_name("foo"),
-            "internal:testinternal:foo",
+            SampleInternal.prefix_with_task_name("foo"),
+            "internal:sampleinternal:foo",
         )
 
-        class TestWorkflow(BaseTask[BaseTaskData, BaseDynamicTaskData]):
+        class SampleWorkflow(BaseTask[BaseTaskData, BaseDynamicTaskData]):
             TASK_TYPE = TaskTypes.WORKFLOW
 
             def _execute(self) -> bool:
@@ -260,11 +253,11 @@ class BaseTaskTests(TestCase):
                 raise NotImplementedError()
 
         self.assertEqual(
-            TestWorkflow.prefix_with_task_name("foo"),
-            "workflow:testworkflow:foo",
+            SampleWorkflow.prefix_with_task_name("foo"),
+            "workflow:sampleworkflow:foo",
         )
 
-        class TestSigning(BaseTask[BaseTaskData, BaseDynamicTaskData]):
+        class SampleSigning(BaseTask[BaseTaskData, BaseDynamicTaskData]):
             TASK_TYPE = TaskTypes.SIGNING
 
             def _execute(self) -> bool:
@@ -274,11 +267,11 @@ class BaseTaskTests(TestCase):
                 raise NotImplementedError()
 
         self.assertEqual(
-            TestSigning.prefix_with_task_name("foo"),
-            "signing:testsigning:foo",
+            SampleSigning.prefix_with_task_name("foo"),
+            "signing:samplesigning:foo",
         )
 
-        class TestWait(BaseTask[BaseTaskData, BaseDynamicTaskData]):
+        class SampleWait(BaseTask[BaseTaskData, BaseDynamicTaskData]):
             TASK_TYPE = TaskTypes.WAIT
 
             def _execute(self) -> bool:
@@ -288,7 +281,7 @@ class BaseTaskTests(TestCase):
                 raise NotImplementedError()
 
         self.assertEqual(
-            TestWait.prefix_with_task_name("foo"), "wait:testwait:foo"
+            SampleWait.prefix_with_task_name("foo"), "wait:samplewait:foo"
         )
 
     def test_get_subject(self) -> None:
@@ -309,36 +302,36 @@ class BaseTaskTests(TestCase):
             "foo": "bar",
         }
 
-        data = TestBaseTask2Data.parse_obj(task_data)
+        data = SampleBaseTask2Data.parse_obj(task_data)
 
         self.assertEqual(
             data,
-            TestBaseTask2Data(
+            SampleBaseTask2Data(
                 foo="bar",
             ),
         )
 
     def test_configure_task_data_notification_drop(self) -> None:
         """Raise TaskConfigError: "notifications" is not valid anymore."""
-        task_data = TestBaseTask2Data(foo="bar")
+        task_data = SampleBaseTask2Data(foo="bar")
         task_data_notification = {
             "notifications": {"on_failure": [{"channel": "test"}]}
         }
 
         with self.assertRaisesRegex(TaskConfigError, "notifications"):
-            TestBaseTask2({**task_data.dict(), **task_data_notification})
+            SampleBaseTask2({**task_data.dict(), **task_data_notification})
 
     def test_configure_fails_with_bad_schema(self) -> None:
         """configure() raises TaskConfigError if schema is not respected."""
         task_data = {"nonexistent": "bar"}
         with self.assertRaises(TaskConfigError):
-            TestBaseTask2(task_data)
+            SampleBaseTask2(task_data)
 
     def test_configure_works_with_good_schema(self) -> None:
         """configure() doesn't raise TaskConfigError."""
-        task_data = TestBaseTask2Data(foo="bar", bar=3.14)
+        task_data = SampleBaseTask2Data(foo="bar", bar=3.14)
 
-        task2 = TestBaseTask2(task_data.dict())
+        task2 = SampleBaseTask2(task_data.dict())
 
         self.assertEqual(task2.data.foo, "bar")
         self.assertEqual(task2.data.bar, 3.14)
@@ -346,12 +339,12 @@ class BaseTaskTests(TestCase):
     def test_analyze_worker_without_task_version(self) -> None:
         """analyze_worker() reports an unknown task version."""
         metadata = self.task.analyze_worker()
-        self.assertIsNone(metadata["testbasetask1:version"])
+        self.assertIsNone(metadata["samplebasetask1:version"])
 
     def test_analyze_worker_with_task_version(self) -> None:
         """analyze_worker() reports a task version."""
         metadata = self.task2.analyze_worker()
-        self.assertEqual(metadata["testbasetask2:version"], 1)
+        self.assertEqual(metadata["samplebasetask2:version"], 1)
 
     def test_analyze_worker_all_tasks(self) -> None:
         """analyze_worker_all_tasks() reports results for each task."""
@@ -360,8 +353,8 @@ class BaseTaskTests(TestCase):
             "_sub_tasks",
             new={
                 TaskTypes.WORKER: {
-                    "task": TestBaseTask1,
-                    "task2": TestBaseTask2,
+                    "task": SampleBaseTask1,
+                    "task2": SampleBaseTask2,
                 }
             },
         )
@@ -432,13 +425,13 @@ class BaseTaskTests(TestCase):
         with self.assertRaisesRegex(AssertionError, msg):
 
             class Somesubclassname(
-                TestBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
+                SampleBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
             ):
                 def run(self, execute_directory: Path) -> bool:  # noqa: U100
                     raise NotImplementedError()
 
             class SomeSubclassName(
-                TestBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
+                SampleBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
             ):
                 def run(self, execute_directory: Path) -> bool:  # noqa: U100
                     raise NotImplementedError()
@@ -450,7 +443,7 @@ class BaseTaskTests(TestCase):
         with self.assertRaisesRegex(AssertionError, msg):
 
             class Somesubclassname(
-                TestBaseTask[BaseTaskData, BaseDynamicTaskData]
+                SampleBaseTask[BaseTaskData, BaseDynamicTaskData]
             ):
                 TASK_TYPE = TaskTypes.SERVER
 
@@ -461,7 +454,7 @@ class BaseTaskTests(TestCase):
                     raise NotImplementedError()
 
             class SomeSubclassName(
-                TestBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
+                SampleBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
             ):
                 def run(self, execute_directory: Path) -> bool:  # noqa: U100
                     raise NotImplementedError()
@@ -470,13 +463,13 @@ class BaseTaskTests(TestCase):
         with self.assertRaisesRegex(AssertionError, msg):
 
             class SomeSubclassName1(
-                TestBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
+                SampleBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
             ):
                 def run(self, execute_directory: Path) -> bool:  # noqa: U100
                     raise NotImplementedError()
 
             class Somesubclassname1(
-                TestBaseTask[BaseTaskData, BaseDynamicTaskData]
+                SampleBaseTask[BaseTaskData, BaseDynamicTaskData]
             ):
                 TASK_TYPE = TaskTypes.SERVER
 
@@ -749,25 +742,21 @@ class BaseTaskTests(TestCase):
         )
 
 
-class TestBaseExternalTask1(
-    TestBaseExternalTask[BaseTaskData, BaseDynamicTaskData]
+class SampleBaseExternalTask1(
+    SampleBaseExternalTask[BaseTaskData, BaseDynamicTaskData]
 ):
     """Sample class to test BaseExternalTask class."""
 
-    __test__ = False
-
     def run(self, execute_directory: Path) -> bool:
         """Unused abstract method from BaseExternalTask."""
         raise NotImplementedError()
 
 
-class TestBaseExternalTask2(
-    TestBaseExternalTask[TestBaseTask2Data, BaseDynamicTaskData]
+class SampleBaseExternalTask2(
+    SampleBaseExternalTask[SampleBaseTask2Data, BaseDynamicTaskData]
 ):
     """Test BaseExternalTask class with data validation."""
 
-    __test__ = False
-
     TASK_VERSION = 1
 
     def run(self, execute_directory: Path) -> bool:
@@ -776,18 +765,20 @@ class TestBaseExternalTask2(
 
 
 class BaseExternalTaskTests(TestCase):
-    """Unit tests for :class:`BaseExternalTask`."""
+    """Unit tests for :py:class:`BaseExternalTask`."""
 
     def setUp(self) -> None:
         """Create the shared attributes."""
-        self.task = TestBaseExternalTask1({})
-        self.task2 = TestBaseExternalTask2({"foo": "bar"})
+        super().setUp()
+        self.task = SampleBaseExternalTask1({})
+        self.task2 = SampleBaseExternalTask2({"foo": "bar"})
         self.worker_metadata = {"system:worker_type": WorkerType.EXTERNAL}
 
     def tearDown(self) -> None:
         """Delete temporary directory, if it exists."""
         if self.task._debug_log_files_directory:
             self.task._debug_log_files_directory.cleanup()
+        super().tearDown()
 
     def test_can_run_on_no_version(self) -> None:
         """Ensure can_run_on returns True if no version is specified."""
@@ -799,7 +790,7 @@ class BaseExternalTaskTests(TestCase):
         """Ensure can_run_on returns False if versions differ."""
         self.assertIsNone(self.task.TASK_VERSION)
         metadata = {**self.worker_metadata, **self.task.analyze_worker()}
-        metadata["testbaseexternaltask1:version"] = 1
+        metadata["samplebaseexternaltask1:version"] = 1
         self.assertFalse(self.task.can_run_on(metadata))
 
     def test_can_run_on_type_mismatch(self) -> None:
@@ -844,7 +835,7 @@ class BaseExternalTaskTests(TestCase):
 
         self.task.work_request_id = 1
         self.task.debusine = mock.create_autospec(spec=Debusine)
-        artifact_response = ArtifactResponse(
+        artifact_response = create_artifact_response(
             workspace=artifact_workspace,
             id=5,
             category=ArtifactCategory.TEST,
@@ -1302,7 +1293,7 @@ class TestsTaskConfigError(TestCase):
 
 
 class RunCommandTaskForTesting(
-    TestBaseTask[BaseTaskData, BaseDynamicTaskData],
+    SampleBaseTask[BaseTaskData, BaseDynamicTaskData],
     RunCommandTask[BaseTaskData, BaseDynamicTaskData],
 ):
     """Used in TaskTests."""
@@ -1448,6 +1439,7 @@ class RunCommandTaskTests(
 
     def setUp(self) -> None:
         """Set up tests."""
+        super().setUp()
         self.temp_directory = self.create_temporary_directory()
         self.task = RunCommandTaskForTesting()
 
@@ -1455,6 +1447,7 @@ class RunCommandTaskTests(
         """Delete temporary directory, if it exists."""
         if self.task._debug_log_files_directory:
             self.task._debug_log_files_directory.cleanup()
+        super().tearDown()
 
     def test_cmdline_as_root(self) -> None:
         """_cmdline_as_root return False."""
@@ -1995,12 +1988,14 @@ class BaseTaskWithExecutorTests(TestCase):
 
     def setUp(self) -> None:
         """Create the shared attributes."""
-        self.task = TestBaseTaskWithExecutor1({})
+        super().setUp()
+        self.task = SampleBaseTaskWithExecutor1({})
 
     def tearDown(self) -> None:
         """Delete directory to avoid ResourceWarning."""
         if self.task._debug_log_files_directory:
             self.task._debug_log_files_directory.cleanup()
+        super().tearDown()
 
     def patch_executor(
         self, backend: BackendType, image_category: ExecutorImageCategory
@@ -2057,7 +2052,144 @@ class BaseTaskWithExecutorTests(TestCase):
 
     def test_get_environment_with_architecture_and_backend(self) -> None:
         """`get_environment` fills in architecture/backend."""
-        task = TestBaseTaskWithExecutorAndArchitecture(
+        task = SampleBaseTaskWithExecutorAndArchitecture(
+            {
+                "backend": BackendType.UNSHARE,
+                "environment": "debian/match:codename=bookworm",
+                "host_architecture": "amd64",
+            }
+        )
+        assert task.data.environment
+        self.patch_executor(BackendType.UNSHARE, ExecutorImageCategory.TARBALL)
+        artifact_info = ArtifactInfo(
+            id=2,
+            category=ArtifactCategory.SYSTEM_TARBALL,
+            data=create_system_tarball_data(),
+        )
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare:variant=",
+                    None,
+                ): artifact_info
+            }
+        )
+
+        self.assertEqual(
+            task.get_environment(task_db, task.data.environment), artifact_info
+        )
+
+    def test_get_environment_with_architecture_from_worker(self) -> None:
+        """`get_environment` falls back to the worker's host architecture."""
+        task = SampleBaseTaskWithExecutorAndArchitecture(
+            {
+                "backend": BackendType.UNSHARE,
+                "environment": "debian/match:codename=bookworm",
+            }
+        )
+        assert task.data.environment
+        task.worker_host_architecture = "amd64"
+        self.patch_executor(BackendType.UNSHARE, ExecutorImageCategory.IMAGE)
+        artifact_info = ArtifactInfo(
+            id=3,
+            category=ArtifactCategory.SYSTEM_TARBALL,
+            data=create_system_tarball_data(),
+        )
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=image:backend=unshare:variant=",
+                    None,
+                ): artifact_info
+            }
+        )
+
+        self.assertEqual(
+            task.get_environment(task_db, task.data.environment), artifact_info
+        )
+
+    def test_get_environment_with_architecture_missing(self) -> None:
+        """`get_environment` skips architecture if it's missing."""
+        task = SampleBaseTaskWithExecutorAndArchitecture(
+            {
+                "backend": BackendType.UNSHARE,
+                "environment": "debian/match:codename=bookworm",
+            }
+        )
+        assert task.data.environment
+        self.patch_executor(BackendType.UNSHARE, ExecutorImageCategory.IMAGE)
+        artifact_info = ArtifactInfo(
+            id=3,
+            category=ArtifactCategory.SYSTEM_TARBALL,
+            data=create_system_tarball_data(),
+        )
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                (
+                    "debian/match:codename=bookworm:format=image:"
+                    "backend=unshare:variant=",
+                    None,
+                ): artifact_info
+            }
+        )
+
+        self.assertEqual(
+            task.get_environment(task_db, task.data.environment), artifact_info
+        )
+
+    def test_get_environment_tries_variant(self) -> None:
+        """`get_environment` tries the task name as a variant first."""
+        task = SampleBaseTaskWithExecutorAndArchitecture(
+            {
+                "backend": BackendType.UNSHARE,
+                "environment": "debian/match:codename=bookworm",
+                "host_architecture": "amd64",
+            }
+        )
+        assert task.data.environment
+        self.patch_executor(BackendType.UNSHARE, ExecutorImageCategory.TARBALL)
+        artifact_info = ArtifactInfo(
+            id=2,
+            category=ArtifactCategory.SYSTEM_TARBALL,
+            data=create_system_tarball_data(),
+        )
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare:"
+                    "variant=samplebasetaskwithexecutorandarchitecture",
+                    None,
+                ): artifact_info,
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare:variant=",
+                    None,
+                ): None,
+            }
+        )
+
+        self.assertEqual(
+            task.get_environment(task_db, task.data.environment), artifact_info
+        )
+
+        task_db.single_lookups = {
+            (
+                "debian/match:codename=bookworm:architecture=amd64:"
+                "format=tarball:backend=unshare:variant=",
+                None,
+            ): artifact_info
+        }
+
+        self.assertEqual(
+            task.get_environment(task_db, task.data.environment), artifact_info
+        )
+
+    def test_get_environment_without_try_variant(self) -> None:
+        """`get_environment` can be told not to try variants."""
+        task = SampleBaseTaskWithExecutorAndArchitecture(
             {
                 "backend": BackendType.UNSHARE,
                 "environment": "debian/match:codename=bookworm",
@@ -2077,71 +2209,15 @@ class BaseTaskWithExecutorTests(TestCase):
                     "debian/match:codename=bookworm:architecture=amd64:"
                     "format=tarball:backend=unshare",
                     None,
-                ): artifact_info
+                ): artifact_info,
             }
         )
 
         self.assertEqual(
-            task.get_environment(task_db, task.data.environment), artifact_info
-        )
-
-    def test_get_environment_with_architecture_from_worker(self) -> None:
-        """`get_environment` falls back to the worker's host architecture."""
-        task = TestBaseTaskWithExecutorAndArchitecture(
-            {
-                "backend": BackendType.UNSHARE,
-                "environment": "debian/match:codename=bookworm",
-            }
-        )
-        assert task.data.environment
-        task.worker_host_architecture = "amd64"
-        self.patch_executor(BackendType.UNSHARE, ExecutorImageCategory.IMAGE)
-        artifact_info = ArtifactInfo(
-            id=3,
-            category=ArtifactCategory.SYSTEM_TARBALL,
-            data=create_system_tarball_data(),
-        )
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=image:backend=unshare",
-                    None,
-                ): artifact_info
-            }
-        )
-
-        self.assertEqual(
-            task.get_environment(task_db, task.data.environment), artifact_info
-        )
-
-    def test_get_environment_with_architecture_missing(self) -> None:
-        """`get_environment` skips architecture if it's missing."""
-        task = TestBaseTaskWithExecutorAndArchitecture(
-            {
-                "backend": BackendType.UNSHARE,
-                "environment": "debian/match:codename=bookworm",
-            }
-        )
-        assert task.data.environment
-        self.patch_executor(BackendType.UNSHARE, ExecutorImageCategory.IMAGE)
-        artifact_info = ArtifactInfo(
-            id=3,
-            category=ArtifactCategory.SYSTEM_TARBALL,
-            data=create_system_tarball_data(),
-        )
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                (
-                    "debian/match:codename=bookworm:format=image:"
-                    "backend=unshare",
-                    None,
-                ): artifact_info
-            }
-        )
-
-        self.assertEqual(
-            task.get_environment(task_db, task.data.environment), artifact_info
+            task.get_environment(
+                task_db, task.data.environment, try_variant=False
+            ),
+            artifact_info,
         )
 
     def test_prepare_executor(self) -> None:
@@ -2282,7 +2358,7 @@ class TestTestTaskMixin(TestCase):
 
     def test_label(self) -> None:
         """Test get_label."""
-        task = TestBaseTask1({})
+        task = SampleBaseTask1({})
         self.assertEqual(task.get_label(), "test")
 
 
@@ -2291,7 +2367,7 @@ class DefaultDynamicDataTest(TestCase):
 
     def test_dynamic_data(self) -> None:
         """Test build_dynamic_data."""
-        task = TestDefaultDynamicData({})
+        task = SampleDefaultDynamicData({})
         self.assertEqual(
             task.compute_dynamic_data(FakeTaskDatabase()), BaseDynamicTaskData()
         )
@@ -2303,7 +2379,7 @@ class ExtraRepositoryMixinTests(TestCase):
     def setUp(self) -> None:
         """Create the shared attributes."""
         super().setUp()
-        self.task = TestExtraRepositoryMixin({})
+        self.task = SampleExtraRepositoryMixin({})
         self.directory = self.create_temporary_directory()
 
     def configure_extra_repositories(
@@ -2499,3 +2575,7 @@ class ExtraRepositoryMixinTests(TestCase):
                 ),
             ],
         )
+
+    def test_get_input_artifacts_ids(self) -> None:
+        """Test get_input_artifacts_ids."""
+        self.assertEqual(self.task.get_input_artifacts_ids(), [])
