@@ -63,7 +63,7 @@ except ImportError:
 
 import tenacity
 
-from debusine.artifacts.models import RuntimeStatistics
+from debusine.artifacts.models import RuntimeStatistics, TaskTypes
 from debusine.client.debusine import Debusine
 from debusine.client.exceptions import TokenDisabledError
 from debusine.client.models import WorkRequestResponse
@@ -74,12 +74,7 @@ from debusine.tasks import (
     TaskConfigError,
 )
 from debusine.tasks.executors import analyze_worker_all_executors
-from debusine.tasks.models import (
-    OutputData,
-    OutputDataError,
-    TaskTypes,
-    WorkerType,
-)
+from debusine.tasks.models import OutputData, OutputDataError, WorkerType
 from debusine.worker.config import ConfigHandler
 from debusine.worker.debusine_async_http_client import DebusineAsyncHttpClient
 from debusine.worker.system_information import (
