@@ -36,13 +36,10 @@ from multidict import CIMultiDict, CIMultiDictProxy
 from yarl import URL
 
 import debusine.tasks.models as task_models
-from debusine.artifacts.models import RuntimeStatistics
+from debusine.artifacts.models import RuntimeStatistics, TaskTypes
 from debusine.client.debusine import Debusine
 from debusine.client.exceptions import TokenDisabledError
-from debusine.client.models import (
-    WorkRequestResponse,
-    model_to_json_serializable_dict,
-)
+from debusine.client.models import model_to_json_serializable_dict
 from debusine.tasks import (
     BaseTask,
     BaseTaskWithExecutor,
@@ -57,12 +54,17 @@ from debusine.tasks.models import (
     OutputDataError,
     SbuildData,
     SbuildInput,
-    TaskTypes,
     WorkerType,
 )
-from debusine.tasks.tests.helper_mixin import TestBaseExternalTask, TestBaseTask
+from debusine.tasks.tests.helper_mixin import (
+    SampleBaseExternalTask,
+    SampleBaseTask,
+)
 from debusine.test import TestCase
-from debusine.test.test_utils import create_artifact_response
+from debusine.test.test_utils import (
+    create_artifact_response,
+    create_work_request_response,
+)
 from debusine.worker import Worker
 from debusine.worker.config import ConfigHandler
 from debusine.worker.system_information import (
@@ -1103,7 +1105,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             pass
 
         class ServerTest(
-            TestBaseTask[ServerTestData, task_models.BaseDynamicTaskData],
+            SampleBaseTask[ServerTestData, task_models.BaseDynamicTaskData],
         ):
             TASK_VERSION = 1
             TASK_TYPE = TaskTypes.SERVER
@@ -1118,7 +1120,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             BaseTask._sub_tasks[TaskTypes.SERVER].__delitem__, "servertest"
         )
 
-        next_work_request = WorkRequestResponse(
+        next_work_request = create_work_request_response(
             id=52,
             task_type="Server",
             task_name="servertest",
@@ -1151,7 +1153,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             pass
 
         class SigningTest(
-            TestBaseExternalTask[
+            SampleBaseExternalTask[
                 SigningTestData, task_models.BaseDynamicTaskData
             ],
         ):
@@ -1165,7 +1167,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             BaseTask._sub_tasks[TaskTypes.SIGNING].__delitem__, "signingtest"
         )
 
-        next_work_request = WorkRequestResponse(
+        next_work_request = create_work_request_response(
             id=52,
             task_type="Signing",
             task_name="signingtest",
@@ -1194,7 +1196,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
 
     async def test_worker_task_on_signing_worker(self) -> None:
         """A signing worker refuses to run a worker task."""
-        next_work_request = WorkRequestResponse(
+        next_work_request = create_work_request_response(
             id=52,
             task_type="Worker",
             task_name="noop",
@@ -1229,7 +1231,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             pass
 
         class SigningTest(
-            TestBaseExternalTask[
+            SampleBaseExternalTask[
                 SigningTestData, task_models.BaseDynamicTaskData
             ],
         ):
@@ -1243,7 +1245,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             BaseTask._sub_tasks[TaskTypes.SIGNING].__delitem__, "signingtest"
         )
 
-        next_work_request = WorkRequestResponse(
+        next_work_request = create_work_request_response(
             id=52,
             task_type="Signing",
             task_name="signingtest",
@@ -1275,7 +1277,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
 
     async def test_unknown_task_name(self) -> None:
         """A worker refuses to run a task with an unknown name."""
-        next_work_request = WorkRequestResponse(
+        next_work_request = create_work_request_response(
             id=52,
             task_type="Worker",
             task_name="nonexistent",
